@@ -1,18 +1,40 @@
+from __future__ import unicode_literals
+
 import ctypes
+import json
 import random
-import unittest
-from django.contrib.gis.geos import *
-from django.contrib.gis.geos.base import gdal, numpy, GEOSBase
-from django.contrib.gis.geos.libgeos import GEOS_PREPARE
+from binascii import a2b_hex, b2a_hex
+from io import BytesIO
+
+from django.contrib.gis.gdal import HAS_GDAL
+
+from django.contrib.gis import memoryview
 from django.contrib.gis.geometry.test_data import TestDataMixin
 
+from django.utils.encoding import force_bytes
+from django.utils import six
+from django.utils.six.moves import xrange
+from django.utils import unittest
+from django.utils.unittest import skipUnless
+
+from .. import HAS_GEOS
+
+if HAS_GEOS:
+    from .. import (GEOSException, GEOSIndexError, GEOSGeometry,
+        GeometryCollection, Point, MultiPoint, Polygon, MultiPolygon, LinearRing,
+        LineString, MultiLineString, fromfile, fromstr, geos_version_info,
+        GEOS_PREPARE)
+    from ..base import gdal, numpy, GEOSBase
+
+
+@skipUnless(HAS_GEOS, "Geos is required.")
 class GEOSTest(unittest.TestCase, TestDataMixin):
 
     @property
     def null_srid(self):
         """
         Returns the proper null SRID depending on the GEOS version.
-        See the comments in `test15_srid` for more details.
+        See the comments in `test_srid` for more details.
         """
         info = geos_version_info()
         if info['version'] == '3.0.0' and info['release_candidate']:
@@ -20,7 +42,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         else:
             return None
 
-    def test00_base(self):
+    def test_base(self):
         "Tests out the GEOSBase class."
         # Testing out GEOSBase class, which provides a `ptr` property
         # that abstracts out access to underlying C pointers.
@@ -56,41 +78,41 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         # result in a TypeError when trying to assign it to the `ptr` property.
         # Thus, memmory addresses (integers) and pointers of the incorrect type
         # (in `bad_ptrs`) will not be allowed.
-        bad_ptrs = (5, ctypes.c_char_p('foobar'))
+        bad_ptrs = (5, ctypes.c_char_p(b'foobar'))
         for bad_ptr in bad_ptrs:
             # Equivalent to `fg.ptr = bad_ptr`
             self.assertRaises(TypeError, fg1._set_ptr, bad_ptr)
             self.assertRaises(TypeError, fg2._set_ptr, bad_ptr)
 
-    def test01a_wkt(self):
+    def test_wkt(self):
         "Testing WKT output."
         for g in self.geometries.wkt_out:
             geom = fromstr(g.wkt)
-            self.assertEqual(g.ewkt, geom.wkt)
+            if geom.hasz and geos_version_info()['version'] >= '3.3.0':
+                self.assertEqual(g.ewkt, geom.wkt)
 
-    def test01b_hex(self):
+    def test_hex(self):
         "Testing HEX output."
         for g in self.geometries.hex_wkt:
             geom = fromstr(g.wkt)
-            self.assertEqual(g.hex, geom.hex)
+            self.assertEqual(g.hex, geom.hex.decode())
 
-    def test01b_hexewkb(self):
+    def test_hexewkb(self):
         "Testing (HEX)EWKB output."
-        from binascii import a2b_hex
-
         # For testing HEX(EWKB).
-        ogc_hex = '01010000000000000000000000000000000000F03F'
+        ogc_hex = b'01010000000000000000000000000000000000F03F'
+        ogc_hex_3d = b'01010000800000000000000000000000000000F03F0000000000000040'
         # `SELECT ST_AsHEXEWKB(ST_GeomFromText('POINT(0 1)', 4326));`
-        hexewkb_2d = '0101000020E61000000000000000000000000000000000F03F'
+        hexewkb_2d = b'0101000020E61000000000000000000000000000000000F03F'
         # `SELECT ST_AsHEXEWKB(ST_GeomFromEWKT('SRID=4326;POINT(0 1 2)'));`
-        hexewkb_3d = '01010000A0E61000000000000000000000000000000000F03F0000000000000040'
+        hexewkb_3d = b'01010000A0E61000000000000000000000000000000000F03F0000000000000040'
 
         pnt_2d = Point(0, 1, srid=4326)
         pnt_3d = Point(0, 1, 2, srid=4326)
 
-        # OGC-compliant HEX will not have SRID nor Z value.
+        # OGC-compliant HEX will not have SRID value.
         self.assertEqual(ogc_hex, pnt_2d.hex)
-        self.assertEqual(ogc_hex, pnt_3d.hex)
+        self.assertEqual(ogc_hex_3d, pnt_3d.hex)
 
         # HEXEWKB should be appropriate for its dimension -- have to use an
         # a WKBWriter w/dimension set accordingly, else GEOS will insert
@@ -110,9 +132,9 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
                 self.fail('Should have raised GEOSException.')
 
         # Same for EWKB.
-        self.assertEqual(buffer(a2b_hex(hexewkb_2d)), pnt_2d.ewkb)
+        self.assertEqual(memoryview(a2b_hex(hexewkb_2d)), pnt_2d.ewkb)
         if GEOS_PREPARE:
-            self.assertEqual(buffer(a2b_hex(hexewkb_3d)), pnt_3d.ewkb)
+            self.assertEqual(memoryview(a2b_hex(hexewkb_3d)), pnt_3d.ewkb)
         else:
             try:
                 ewkb = pnt_3d.ewkb
@@ -124,27 +146,22 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         # Redundant sanity check.
         self.assertEqual(4326, GEOSGeometry(hexewkb_2d).srid)
 
-    def test01c_kml(self):
+    def test_kml(self):
         "Testing KML output."
         for tg in self.geometries.wkt_out:
             geom = fromstr(tg.wkt)
             kml = getattr(tg, 'kml', False)
             if kml: self.assertEqual(kml, geom.kml)
 
-    def test01d_errors(self):
+    def test_errors(self):
         "Testing the Error handlers."
         # string-based
-        print "\nBEGIN - expecting GEOS_ERROR; safe to ignore.\n"
         for err in self.geometries.errors:
-            try:
-                g = fromstr(err.wkt)
-            except (GEOSException, ValueError):
-                pass
+            with self.assertRaises((GEOSException, ValueError)):
+                _ = fromstr(err.wkt)
 
         # Bad WKB
-        self.assertRaises(GEOSException, GEOSGeometry, buffer('0'))
-
-        print "\nEND - expecting GEOS_ERROR; safe to ignore.\n"
+        self.assertRaises(GEOSException, GEOSGeometry, memoryview(b'0'))
 
         class NotAGeometry(object):
             pass
@@ -154,15 +171,14 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         # None
         self.assertRaises(TypeError, GEOSGeometry, None)
 
-    def test01e_wkb(self):
+    def test_wkb(self):
         "Testing WKB output."
-        from binascii import b2a_hex
         for g in self.geometries.hex_wkt:
             geom = fromstr(g.wkt)
             wkb = geom.wkb
-            self.assertEqual(b2a_hex(wkb).upper(), g.hex)
+            self.assertEqual(b2a_hex(wkb).decode().upper(), g.hex)
 
-    def test01f_create_hex(self):
+    def test_create_hex(self):
         "Testing creation from HEX."
         for g in self.geometries.hex_wkt:
             geom_h = GEOSGeometry(g.hex)
@@ -170,17 +186,16 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             geom_t = fromstr(g.wkt)
             self.assertEqual(geom_t.wkt, geom_h.wkt)
 
-    def test01g_create_wkb(self):
+    def test_create_wkb(self):
         "Testing creation from WKB."
-        from binascii import a2b_hex
         for g in self.geometries.hex_wkt:
-            wkb = buffer(a2b_hex(g.hex))
+            wkb = memoryview(a2b_hex(g.hex.encode()))
             geom_h = GEOSGeometry(wkb)
             # we need to do this so decimal places get normalised
             geom_t = fromstr(g.wkt)
             self.assertEqual(geom_t.wkt, geom_h.wkt)
 
-    def test01h_ewkt(self):
+    def test_ewkt(self):
         "Testing EWKT."
         srids = (-1, 32140)
         for srid in srids:
@@ -191,25 +206,25 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
                 self.assertEqual(srid, poly.shell.srid)
                 self.assertEqual(srid, fromstr(poly.ewkt).srid) # Checking export
 
-    def test01i_json(self):
+    @skipUnless(HAS_GDAL, "GDAL is required.")
+    def test_json(self):
         "Testing GeoJSON input/output (via GDAL)."
-        if not gdal or not gdal.GEOJSON: return
         for g in self.geometries.json_geoms:
             geom = GEOSGeometry(g.wkt)
             if not hasattr(g, 'not_equal'):
-                self.assertEqual(g.json, geom.json)
-                self.assertEqual(g.json, geom.geojson)
+                # Loading jsons to prevent decimal differences
+                self.assertEqual(json.loads(g.json), json.loads(geom.json))
+                self.assertEqual(json.loads(g.json), json.loads(geom.geojson))
             self.assertEqual(GEOSGeometry(g.wkt), GEOSGeometry(geom.json))
 
-    def test01k_fromfile(self):
+    def test_fromfile(self):
         "Testing the fromfile() factory."
-        from StringIO import StringIO
         ref_pnt = GEOSGeometry('POINT(5 23)')
 
-        wkt_f = StringIO()
-        wkt_f.write(ref_pnt.wkt)
-        wkb_f = StringIO()
-        wkb_f.write(str(ref_pnt.wkb))
+        wkt_f = BytesIO()
+        wkt_f.write(force_bytes(ref_pnt.wkt))
+        wkb_f = BytesIO()
+        wkb_f.write(bytes(ref_pnt.wkb))
 
         # Other tests use `fromfile()` on string filenames so those
         # aren't tested here.
@@ -218,7 +233,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             pnt = fromfile(fh)
             self.assertEqual(ref_pnt, pnt)
 
-    def test01k_eq(self):
+    def test_eq(self):
         "Testing equivalence."
         p = fromstr('POINT(5 23)')
         self.assertEqual(p, p.wkt)
@@ -233,7 +248,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             self.assertNotEqual(g, {'foo' : 'bar'})
             self.assertNotEqual(g, False)
 
-    def test02a_points(self):
+    def test_points(self):
         "Testing Point objects."
         prev = fromstr('POINT(0 0)')
         for p in self.geometries.points:
@@ -288,7 +303,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
 
             prev = pnt # setting the previous geometry
 
-    def test02b_multipoints(self):
+    def test_multipoints(self):
         "Testing MultiPoint objects."
         for mp in self.geometries.multipoints:
             mpnt = fromstr(mp.wkt)
@@ -307,7 +322,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
                 self.assertEqual(p.empty, False)
                 self.assertEqual(p.valid, True)
 
-    def test03a_linestring(self):
+    def test_linestring(self):
         "Testing LineString objects."
         prev = fromstr('POINT(0 0)')
         for l in self.geometries.linestrings:
@@ -333,7 +348,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             self.assertEqual(ls.wkt, LineString(*tuple(Point(tup) for tup in ls.tuple)).wkt) # Point individual arguments
             if numpy: self.assertEqual(ls, LineString(numpy.array(ls.tuple))) # as numpy array
 
-    def test03b_multilinestring(self):
+    def test_multilinestring(self):
         "Testing MultiLineString objects."
         prev = fromstr('POINT(0 0)')
         for l in self.geometries.multilinestrings:
@@ -357,7 +372,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             self.assertEqual(ml.wkt, MultiLineString(*tuple(s.clone() for s in ml)).wkt)
             self.assertEqual(ml, MultiLineString(*tuple(LineString(s.tuple) for s in ml)))
 
-    def test04_linearring(self):
+    def test_linearring(self):
         "Testing LinearRing objects."
         for rr in self.geometries.linearrings:
             lr = fromstr(rr.wkt)
@@ -373,14 +388,22 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             self.assertEqual(lr, LinearRing([list(tup) for tup in lr.tuple]))
             if numpy: self.assertEqual(lr, LinearRing(numpy.array(lr.tuple)))
 
-    def test05a_polygons(self):
-        "Testing Polygon objects."
-
-        # Testing `from_bbox` class method
+    def test_polygons_from_bbox(self):
+        "Testing `from_bbox` class method."
         bbox = (-180, -90, 180, 90)
-        p = Polygon.from_bbox( bbox )
+        p = Polygon.from_bbox(bbox)
         self.assertEqual(bbox, p.extent)
 
+        # Testing numerical precision
+        x = 3.14159265358979323
+        bbox = (0, 0, 1, x)
+        p = Polygon.from_bbox(bbox)
+        y = p.extent[-1]
+        self.assertEqual(format(x, '.13f'), format(y, '.13f'))
+
+    def test_polygons(self):
+        "Testing Polygon objects."
+
         prev = fromstr('POINT(0 0)')
         for p in self.geometries.polygons:
             # Creating the Polygon, testing its properties.
@@ -422,8 +445,8 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
                 self.assertEqual(r.geom_typeid, 2)
 
             # Testing polygon construction.
-            self.assertRaises(TypeError, Polygon.__init__, 0, [1, 2, 3])
-            self.assertRaises(TypeError, Polygon.__init__, 'foo')
+            self.assertRaises(TypeError, Polygon, 0, [1, 2, 3])
+            self.assertRaises(TypeError, Polygon, 'foo')
 
             # Polygon(shell, (hole1, ... holeN))
             rings = tuple(r for r in poly)
@@ -437,9 +460,23 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             self.assertEqual(poly.wkt, Polygon(*tuple(r for r in poly)).wkt)
             self.assertEqual(poly.wkt, Polygon(*tuple(LinearRing(r.tuple) for r in poly)).wkt)
 
-    def test05b_multipolygons(self):
+    def test_polygon_comparison(self):
+        p1 = Polygon(((0, 0), (0, 1), (1, 1), (1, 0), (0, 0)))
+        p2 = Polygon(((0, 0), (0, 1), (1, 0), (0, 0)))
+        self.assertTrue(p1 > p2)
+        self.assertFalse(p1 < p2)
+        self.assertFalse(p2 > p1)
+        self.assertTrue(p2 < p1)
+
+        p3 = Polygon(((0, 0), (0, 1), (1, 1), (2, 0), (0, 0)))
+        p4 = Polygon(((0, 0), (0, 1), (2, 2), (1, 0), (0, 0)))
+        self.assertFalse(p4 < p3)
+        self.assertTrue(p3 < p4)
+        self.assertTrue(p4 > p3)
+        self.assertFalse(p3 > p4)
+
+    def test_multipolygons(self):
         "Testing MultiPolygon objects."
-        print "\nBEGIN - expecting GEOS_NOTICE; safe to ignore.\n"
         prev = fromstr('POINT (0 0)')
         for mp in self.geometries.multipolygons:
             mpoly = fromstr(mp.wkt)
@@ -458,9 +495,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
                     self.assertEqual(p.valid, True)
                 self.assertEqual(mpoly.wkt, MultiPolygon(*tuple(poly.clone() for poly in mpoly)).wkt)
 
-        print "\nEND - expecting GEOS_NOTICE; safe to ignore.\n"
-
-    def test06a_memory_hijinks(self):
+    def test_memory_hijinks(self):
         "Testing Geometry __del__() on rings and polygons."
         #### Memory issues with rings and polygons
 
@@ -483,7 +518,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         # Access to these rings is OK since they are clones.
         s1, s2 = str(ring1), str(ring2)
 
-    def test08_coord_seq(self):
+    def test_coord_seq(self):
         "Testing Coordinate Sequence objects."
         for p in self.geometries.polygons:
             if p.ext_ring_cs:
@@ -510,7 +545,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
                         cs[i] = tset
                         self.assertEqual(tset[j], cs[i][j])
 
-    def test09_relate_pattern(self):
+    def test_relate_pattern(self):
         "Testing relate() and relate_pattern()."
         g = fromstr('POINT (0 0)')
         self.assertRaises(GEOSException, g.relate_pattern, 0, 'invalid pattern, yo')
@@ -520,7 +555,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             self.assertEqual(rg.result, a.relate_pattern(b, rg.pattern))
             self.assertEqual(rg.pattern, a.relate(b))
 
-    def test10_intersection(self):
+    def test_intersection(self):
         "Testing intersects() and intersection()."
         for i in xrange(len(self.geometries.topology_geoms)):
             a = fromstr(self.geometries.topology_geoms[i].wkt_a)
@@ -533,7 +568,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             a &= b # testing __iand__
             self.assertEqual(i1, a)
 
-    def test11_union(self):
+    def test_union(self):
         "Testing union()."
         for i in xrange(len(self.geometries.topology_geoms)):
             a = fromstr(self.geometries.topology_geoms[i].wkt_a)
@@ -545,7 +580,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             a |= b # testing __ior__
             self.assertEqual(u1, a)
 
-    def test12_difference(self):
+    def test_difference(self):
         "Testing difference()."
         for i in xrange(len(self.geometries.topology_geoms)):
             a = fromstr(self.geometries.topology_geoms[i].wkt_a)
@@ -557,7 +592,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             a -= b # testing __isub__
             self.assertEqual(d1, a)
 
-    def test13_symdifference(self):
+    def test_symdifference(self):
         "Testing sym_difference()."
         for i in xrange(len(self.geometries.topology_geoms)):
             a = fromstr(self.geometries.topology_geoms[i].wkt_a)
@@ -569,7 +604,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             a ^= b # testing __ixor__
             self.assertEqual(d1, a)
 
-    def test14_buffer(self):
+    def test_buffer(self):
         "Testing buffer()."
         for bg in self.geometries.buffer_geoms:
             g = fromstr(bg.wkt)
@@ -597,7 +632,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
                     self.assertAlmostEqual(exp_ring[k][0], buf_ring[k][0], 9)
                     self.assertAlmostEqual(exp_ring[k][1], buf_ring[k][1], 9)
 
-    def test15_srid(self):
+    def test_srid(self):
         "Testing the SRID property and keyword."
         # Testing SRID keyword on Point
         pnt = Point(5, 23, srid=4326)
@@ -635,7 +670,24 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         p3 = fromstr(p1.hex, srid=-1) # -1 is intended.
         self.assertEqual(-1, p3.srid)
 
-    def test16_mutable_geometries(self):
+    @skipUnless(HAS_GDAL, "GDAL is required.")
+    def test_custom_srid(self):
+        """ Test with a srid unknown from GDAL """
+        pnt = Point(111200, 220900, srid=999999)
+        self.assertTrue(pnt.ewkt.startswith("SRID=999999;POINT (111200.0"))
+        self.assertIsInstance(pnt.ogr, gdal.OGRGeometry)
+        self.assertIsNone(pnt.srs)
+
+        # Test conversion from custom to a known srid
+        c2w = gdal.CoordTransform(
+            gdal.SpatialReference('+proj=mill +lat_0=0 +lon_0=0 +x_0=0 +y_0=0 +R_A +ellps=WGS84 +datum=WGS84 +units=m +no_defs'),
+            gdal.SpatialReference(4326))
+        new_pnt = pnt.transform(c2w, clone=True)
+        self.assertEqual(new_pnt.srid, 4326)
+        self.assertAlmostEqual(new_pnt.x, 1, 3)
+        self.assertAlmostEqual(new_pnt.y, 2, 3)
+
+    def test_mutable_geometries(self):
         "Testing the mutability of Polygons and Geometry Collections."
         ### Testing the mutability of Polygons ###
         for p in self.geometries.polygons:
@@ -699,7 +751,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         #self.assertEqual((3.14, 2.71), mpoly[0].shell[0])
         #del mpoly
 
-    def test17_threed(self):
+    def test_threed(self):
         "Testing three-dimensional geometries."
         # Testing a 3D Point
         pnt = Point(2, 3, 8)
@@ -715,7 +767,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         ls[0] = (1.,2.,3.)
         self.assertEqual((1.,2.,3.), ls[0])
 
-    def test18_distance(self):
+    def test_distance(self):
         "Testing the distance() function."
         # Distance to self should be 0.
         pnt = Point(0, 0)
@@ -733,7 +785,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         ls2 = LineString((5, 2), (6, 1), (7, 0))
         self.assertEqual(3, ls1.distance(ls2))
 
-    def test19_length(self):
+    def test_length(self):
         "Testing the length property."
         # Points have 0 length.
         pnt = Point(0, 0)
@@ -751,7 +803,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         mpoly = MultiPolygon(poly.clone(), poly)
         self.assertEqual(8.0, mpoly.length)
 
-    def test20a_emptyCollections(self):
+    def test_emptyCollections(self):
         "Testing empty geometries and collections."
         gc1 = GeometryCollection([])
         gc2 = fromstr('GEOMETRYCOLLECTION EMPTY')
@@ -789,7 +841,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             else:
                 self.assertRaises(GEOSIndexError, g.__getitem__, 0)
 
-    def test20b_collections_of_collections(self):
+    def test_collections_of_collections(self):
         "Testing GeometryCollection handling of other collections."
         # Creating a GeometryCollection WKT string composed of other
         # collections and polygons.
@@ -808,20 +860,25 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         # And, they should be equal.
         self.assertEqual(gc1, gc2)
 
-    def test21_test_gdal(self):
+    @skipUnless(HAS_GDAL, "GDAL is required.")
+    def test_gdal(self):
         "Testing `ogr` and `srs` properties."
-        if not gdal.HAS_GDAL: return
         g1 = fromstr('POINT(5 23)')
-        self.assertEqual(True, isinstance(g1.ogr, gdal.OGRGeometry))
-        self.assertEqual(g1.srs, None)
+        self.assertIsInstance(g1.ogr, gdal.OGRGeometry)
+        self.assertIsNone(g1.srs)
+
+        if GEOS_PREPARE:
+            g1_3d = fromstr('POINT(5 23 8)')
+            self.assertIsInstance(g1_3d.ogr, gdal.OGRGeometry)
+            self.assertEqual(g1_3d.ogr.z, 8)
 
         g2 = fromstr('LINESTRING(0 0, 5 5, 23 23)', srid=4326)
-        self.assertEqual(True, isinstance(g2.ogr, gdal.OGRGeometry))
-        self.assertEqual(True, isinstance(g2.srs, gdal.SpatialReference))
+        self.assertIsInstance(g2.ogr, gdal.OGRGeometry)
+        self.assertIsInstance(g2.srs, gdal.SpatialReference)
         self.assertEqual(g2.hex, g2.ogr.hex)
         self.assertEqual('WGS 84', g2.srs.name)
 
-    def test22_copy(self):
+    def test_copy(self):
         "Testing use with the Python `copy` module."
         import copy
         poly = GEOSGeometry('POLYGON((0 0, 0 23, 23 23, 23 0, 0 0), (5 5, 5 10, 10 10, 10 5, 5 5))')
@@ -830,9 +887,9 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         self.assertNotEqual(poly._ptr, cpy1._ptr)
         self.assertNotEqual(poly._ptr, cpy2._ptr)
 
-    def test23_transform(self):
+    @skipUnless(HAS_GDAL, "GDAL is required to transform geometries")
+    def test_transform(self):
         "Testing `transform` method."
-        if not gdal.HAS_GDAL: return
         orig = GEOSGeometry('POINT (-104.609 38.255)', 4326)
         trans = GEOSGeometry('POINT (992385.4472045 481455.4944650)', 2774)
 
@@ -855,7 +912,17 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             self.assertAlmostEqual(trans.x, p.x, prec)
             self.assertAlmostEqual(trans.y, p.y, prec)
 
-    def test23_transform_noop(self):
+    @skipUnless(HAS_GDAL, "GDAL is required to transform geometries")
+    def test_transform_3d(self):
+        p3d = GEOSGeometry('POINT (5 23 100)', 4326)
+        p3d.transform(2774)
+        if GEOS_PREPARE:
+            self.assertEqual(p3d.z, 100)
+        else:
+            self.assertIsNone(p3d.z)
+
+    @skipUnless(HAS_GDAL, "GDAL is required.")
+    def test_transform_noop(self):
         """ Testing `transform` method (SRID match) """
         # transform() should no-op if source & dest SRIDs match,
         # regardless of whether GDAL is available.
@@ -890,66 +957,23 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         finally:
             gdal.HAS_GDAL = old_has_gdal
 
-    def test23_transform_nosrid(self):
-        """ Testing `transform` method (no SRID) """
-        # Raise a warning if SRID <0/None.
-        import warnings
-        print "\nBEGIN - expecting Warnings; safe to ignore.\n"
+    def test_transform_nosrid(self):
+        """ Testing `transform` method (no SRID or negative SRID) """
 
-        # Test for do-nothing behavior.
-        try:
-            # Keeping line-noise down by only printing the relevant
-            # warnings once.
-            warnings.simplefilter('once', UserWarning)
-            warnings.simplefilter('once', FutureWarning)
+        g = GEOSGeometry('POINT (-104.609 38.255)', srid=None)
+        self.assertRaises(GEOSException, g.transform, 2774)
 
-            g = GEOSGeometry('POINT (-104.609 38.255)', srid=None)
-            g.transform(2774)
-            self.assertEqual(g.tuple, (-104.609, 38.255))
-            self.assertEqual(g.srid, None)
+        g = GEOSGeometry('POINT (-104.609 38.255)', srid=None)
+        self.assertRaises(GEOSException, g.transform, 2774, clone=True)
 
-            g = GEOSGeometry('POINT (-104.609 38.255)', srid=None)
-            g1 = g.transform(2774, clone=True)
-            self.assertTrue(g1 is None)
+        g = GEOSGeometry('POINT (-104.609 38.255)', srid=-1)
+        self.assertRaises(GEOSException, g.transform, 2774)
 
-            g = GEOSGeometry('POINT (-104.609 38.255)', srid=-1)
-            g.transform(2774)
-            self.assertEqual(g.tuple, (-104.609, 38.255))
-            self.assertEqual(g.srid, -1)
+        g = GEOSGeometry('POINT (-104.609 38.255)', srid=-1)
+        self.assertRaises(GEOSException, g.transform, 2774, clone=True)
 
-            g = GEOSGeometry('POINT (-104.609 38.255)', srid=-1)
-            g1 = g.transform(2774, clone=True)
-            self.assertTrue(g1 is None)
-
-        finally:
-            warnings.simplefilter('default', UserWarning)
-            warnings.simplefilter('default', FutureWarning)
-
-        print "\nEND - expecting Warnings; safe to ignore.\n"
-
-
-        # test warning is raised
-        try:
-            warnings.simplefilter('error', FutureWarning)
-            warnings.simplefilter('ignore', UserWarning)
-
-            g = GEOSGeometry('POINT (-104.609 38.255)', srid=None)
-            self.assertRaises(FutureWarning, g.transform, 2774)
-
-            g = GEOSGeometry('POINT (-104.609 38.255)', srid=None)
-            self.assertRaises(FutureWarning, g.transform, 2774, clone=True)
-
-            g = GEOSGeometry('POINT (-104.609 38.255)', srid=-1)
-            self.assertRaises(FutureWarning, g.transform, 2774)
-
-            g = GEOSGeometry('POINT (-104.609 38.255)', srid=-1)
-            self.assertRaises(FutureWarning, g.transform, 2774, clone=True)
-        finally:
-            warnings.simplefilter('default', FutureWarning)
-            warnings.simplefilter('default', UserWarning)
-
-
-    def test23_transform_nogdal(self):
+    @skipUnless(HAS_GDAL, "GDAL is required.")
+    def test_transform_nogdal(self):
         """ Testing `transform` method (GDAL not available) """
         old_has_gdal = gdal.HAS_GDAL
         try:
@@ -963,7 +987,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         finally:
             gdal.HAS_GDAL = old_has_gdal
 
-    def test24_extent(self):
+    def test_extent(self):
         "Testing `extent` method."
         # The xmin, ymin, xmax, ymax of the MultiPoint should be returned.
         mp = MultiPoint(Point(5, 23), Point(0, 0), Point(10, 50))
@@ -979,10 +1003,11 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         xmax, ymax = max(x), max(y)
         self.assertEqual((xmin, ymin, xmax, ymax), poly.extent)
 
-    def test25_pickle(self):
+    def test_pickle(self):
         "Testing pickling and unpickling support."
         # Using both pickle and cPickle -- just 'cause.
-        import pickle, cPickle
+        from django.utils.six.moves import cPickle
+        import pickle
 
         # Creating a list of test geometries for pickling,
         # and setting the SRID on some of them.
@@ -1002,9 +1027,9 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
                 self.assertEqual(geom, tmpg)
                 if not no_srid: self.assertEqual(geom.srid, tmpg.srid)
 
-    def test26_prepared(self):
+    @skipUnless(HAS_GEOS and GEOS_PREPARE, "geos >= 3.1.0 is required")
+    def test_prepared(self):
         "Testing PreparedGeometry support."
-        if not GEOS_PREPARE: return
         # Creating a simple multipolygon and getting a prepared version.
         mpoly = GEOSGeometry('MULTIPOLYGON(((0 0,0 5,5 5,5 0,0 0)),((5 5,5 10,10 10,10 5,5 5)))')
         prep = mpoly.prepared
@@ -1018,7 +1043,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             self.assertEqual(mpoly.intersects(pnt), prep.intersects(pnt))
             self.assertEqual(c, prep.covers(pnt))
 
-    def test26_line_merge(self):
+    def test_line_merge(self):
         "Testing line merge support"
         ref_geoms = (fromstr('LINESTRING(1 1, 1 1, 3 3)'),
                      fromstr('MULTILINESTRING((1 1, 3 3), (3 3, 4 2))'),
@@ -1029,27 +1054,43 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         for geom, merged in zip(ref_geoms, ref_merged):
             self.assertEqual(merged, geom.merged)
 
-    def test27_valid_reason(self):
+    @skipUnless(HAS_GEOS and GEOS_PREPARE, "geos >= 3.1.0 is required")
+    def test_valid_reason(self):
         "Testing IsValidReason support"
-        # Skipping tests if GEOS < v3.1.
-        if not GEOS_PREPARE: return
 
         g = GEOSGeometry("POINT(0 0)")
         self.assertTrue(g.valid)
-        self.assertTrue(isinstance(g.valid_reason, basestring))
+        self.assertIsInstance(g.valid_reason, six.string_types)
         self.assertEqual(g.valid_reason, "Valid Geometry")
 
-        print "\nBEGIN - expecting GEOS_NOTICE; safe to ignore.\n"
-
         g = GEOSGeometry("LINESTRING(0 0, 0 0)")
 
-        self.assertTrue(not g.valid)
-        self.assertTrue(isinstance(g.valid_reason, basestring))
+        self.assertFalse(g.valid)
+        self.assertIsInstance(g.valid_reason, six.string_types)
         self.assertTrue(g.valid_reason.startswith("Too few points in geometry component"))
 
-        print "\nEND - expecting GEOS_NOTICE; safe to ignore.\n"
+    @skipUnless(HAS_GEOS and geos_version_info()['version'] >= '3.2.0', "geos >= 3.2.0 is required")
+    def test_linearref(self):
+        "Testing linear referencing"
 
-    def test28_geos_version(self):
+        ls = fromstr('LINESTRING(0 0, 0 10, 10 10, 10 0)')
+        mls = fromstr('MULTILINESTRING((0 0, 0 10), (10 0, 10 10))')
+
+        self.assertEqual(ls.project(Point(0, 20)), 10.0)
+        self.assertEqual(ls.project(Point(7, 6)), 24)
+        self.assertEqual(ls.project_normalized(Point(0, 20)), 1.0/3)
+
+        self.assertEqual(ls.interpolate(10), Point(0, 10))
+        self.assertEqual(ls.interpolate(24), Point(10, 6))
+        self.assertEqual(ls.interpolate_normalized(1.0/3), Point(0, 10))
+
+        self.assertEqual(mls.project(Point(0, 20)), 10)
+        self.assertEqual(mls.project(Point(7, 6)), 16)
+
+        self.assertEqual(mls.interpolate(9), Point(0, 9))
+        self.assertEqual(mls.interpolate(17), Point(10, 7))
+
+    def test_geos_version(self):
         """Testing the GEOS version regular expression."""
         from django.contrib.gis.geos.libgeos import version_regex
         versions = [('3.0.0rc4-CAPI-1.3.3', '3.0.0', '1.3.3'),
@@ -1061,12 +1102,3 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             self.assertTrue(m, msg="Unable to parse the version string '%s'" % v_init)
             self.assertEqual(m.group('version'), v_geos)
             self.assertEqual(m.group('capi_version'), v_capi)
-
-
-def suite():
-    s = unittest.TestSuite()
-    s.addTest(unittest.makeSuite(GEOSTest))
-    return s
-
-def run(verbosity=2):
-    unittest.TextTestRunner(verbosity=verbosity).run(suite())
